import java.io.*;
import java.awt.*;
import java.util.*;
import com.sun.java.swing.*;
/**
 * This file is an agglomeration of all classes representing the different record
 * types in slog. In this Jumpshot application we actually read only CLOG_HEADER, 
 * CLOG_STATE, CLOG_RAW, CLOG_CNAME and CLOG_DESC records. All other records are
 * skipped over.
 *
 * The function getSize () in all these classes is very important as it 
 * determines the number of bytes that should be skipped.
 */


/**
 * This is a static class which stores all global constants.
 */
class CONST
{
  public static final int TIMELINES = 0;
  public static final int MTN_RANGES = 1;
  public static final String DSTR [] = {"Time Lines", "Mountain Ranges"};
}



class Info
{
    double  begT;
    double  endT;
    double  lenT;
    boolean blink = false;

    public  Info() {}
}

//This class represents each state
class StateInfo extends Info
{ 
    int       groupID;        // Label used in Y axis in timelines window
    int       level = 0;      // For nested states
    StateInfo higher = null;  // Pointer to the next higher state
                              // enveloping this state.
    CLOG_STATE stateDef;
  
    //Constructor 
    public StateInfo() 
    { super(); }
  
    public StateInfo( final SLOG_Irec_min     irec_min,
                      final SLOG_ThreadInfos  thread_infos,
                      final int               view_idx )
    {
        begT    = irec_min.starttime;
        lenT    = irec_min.duration;
        endT    = begT + lenT;
        blink   = false;
        groupID = SetGroupID( irec_min.node_id, irec_min.cpu_id,
                              irec_min.thread_id, thread_infos, view_idx );
    }

    public void SetStateDefLink( CLOG_STATE in_statedef )
    {
        stateDef = in_statedef;
    }

    private int SetGroupID( short node_id, byte cpu_id, int thread_id,
                            final SLOG_ThreadInfos  thread_infos,
                                  int               view_idx )
    // throws IndexOutOfBoundsException
    {
        // The switch statement needs to be sychronized with 
        // argument in the constructor call of ViewFrameChooser in
        // Mainwin.java
        switch (view_idx) {
            case 0:
                return thread_infos.GetAppID( node_id, thread_id );
            case 1:
                return thread_infos.GetOSthreadID( node_id, thread_id );
            case 2:
                return cpu_id;
            default:
                return thread_infos.GetOSprocessID( node_id, thread_id );
        }
    }
  
    public String toString()
    {
        return ( "StateInfo=[ level=" + level + ", groupID=" + groupID + 
                 ", begT=" + begT + ", endT=" + endT + ", lenT=" + lenT +
                 ", name=" + stateDef.description.desc + " ]" );
    }
}



//This class represents a state definition
class CLOG_STATE
{
    int       intvltype;
    String    classtype;
    CLOG_DESC description;  // string describing state
    Color     color;        // Color given to this state
  
    Vector stateVector;     // Vector of completed states (paired up events)

    // This checkbox determines whether states beloging to this state def.,
    // should be displayed or not.
    JCheckBox checkbox;
  
    public CLOG_STATE()
    {
        description = new CLOG_DESC();
        color       = null;
        // stateVector.size() determines if JCheckbox = true 
        // in StateButtons.setupPanels()
        stateVector = new Vector( 0 ); 
    }

    public CLOG_STATE( SLOG_ProfileEntry entry )
    {
        intvltype   = entry.intvltype;
        classtype   = new String( entry.classtype );
        description = new CLOG_DESC( entry.label );
        color       = COLOR_UTIL.getColor( entry.color );
        // stateVector.size() determines if JCheckbox = true 
        // in StateButtons.setupPanels()
        stateVector = new Vector( 0 );
    }
  
    public String toString()
    { 
        return ("CLOG_STATE=[ intvltype=" + intvltype
                            + ", classtype=" + classtype
                            + ", color=" + color.toString ()
                            + ", description=" + description.desc
                            + " ]");
    }
}



class CLOG_DESC
{
    String desc;
  
    //Constructor
    public CLOG_DESC() { desc = new String (); }
  
    //   Copy Constructor
    public CLOG_DESC( String src ) { desc = new String( src ); }

    public int getSize() { return desc.length(); }
  
    public String toString() { return ("CLOG_DESC=[ desc=" + desc + " ]"); }
}



class CLOG_CNAME 
{
    String name;
  
    //Constructor
    public CLOG_CNAME() { name = new String (); }
  
    //   Copy Constructor
    public CLOG_CNAME( String src ) { name = new String( src ); }

    public int getSize() { return name.length(); }

    public String toString() { return ("CLOG_CNAME=[ name=" + name + " ]"); }
}



//This class stores information about a message
class ArrowInfo extends Info
{
  int begGroupID;
  int endGroupID;
  int tag;
  int size;
  
  //Constructor for the object representing a message.
  //bid = Process Id of the process where the message originates
  //eid = Process Id of the process where the message ends.
  //beg = The time at which the message originates.
  //t = Value of tag contained in message.
  public ArrowInfo (int bid, int eid, double beg, double end, int t, int s) {
    super ();
    begGroupID = bid;
    endGroupID = eid;
    begT = beg;
    endT = end;
    tag = t;
    size = s;
  }
  
  public String toString () {
    return ("ArrowInfo [begGroupID=" + begGroupID
                   + ", endGroupID=" + endGroupID
	           + ", begT=" + begT + ", endT=" + endT
                   + ", tag=" + tag + ", size=" + size + "]");
  }
} 



//This object is a repository for all ArrowInfo objects which represent messages
class CLOG_ARROW
{
  Vector sends;         //Temporary vector to store incomplete ArrowInfo objects
                         //arising from unmatched RAW events
  Vector recvs;
  Vector arrowVector;         //Vector that stores completed ArrowInfo objects
  int startetype;        //startetype identifying the start of a message
  int endetype;          //endetype identifying the end of a message
    
  //Constructor
  public CLOG_ARROW () {
    sends = new Vector ();
    recvs = new Vector ();
    arrowVector = new Vector ();
  } 
}



class MtnInfo
{ 
    double begT;
    double endT;
    int []s;
  
    //Constructor 
    public MtnInfo (int n) { s = new int [n]; }
  
    public String toString() 
    {
        return ( "MtnInfo=[ begT=" + begT + ", endT=" + endT
               + ", number = " + s.length + " ]");
  }
}
